# Generic Phenology model

The model aims to be a generic model for phenology prediction of a number of species of plants and insects. It is based on the assumption that the development of the specimen is temperature driven. Its general nature requires the user to provide both weather data and the specific characteristics data for the species/variant at hand.

&copy; 2021 NIBIO, the Norwegian Institute of Bioeconomy Research

**Authors:** Bhabesh Bhabani Mukhopadhyay and Tor-Einar Skog

## Input data:
* Weather data
* Phenhology phases and their heat requirements
* Day degree calculation method

See a full example of input data for Fall Armyworm [here](doc/input_data_example.md)

An exerpt is found below

```json
{
	"plantationData": {
		"speciesLatinName": "FAW",
		"typeName": "GENERIC",
		"baseTemp": 13.0,
		"optTemp": 28.0,
		"upperTemp": 35.0,
		"logIntervalId": 1,
		"algorithmId": 2,
		"startDate": 1605571200000,
		"weatherParams": ["TM"],
		"plantations": [{
			"typeName": "GENERIC",
			"phaseName": "Egg",
			"heatReq": 0.0
		}, {
			"typeName": "GENERIC",
			"phaseName": "Hatched egg",
			"heatReq": 33.35
		}]
	},
	"observations": [{
		"elementMeasurementTypeId": "TM",
		"logIntervalId": 1,
		"timeMeasured": "2020-11-04T08:00:00Z",
		"value": 31.609090909090913
	}

```
Please observe these properties:
* speciesLatinName: Should be (but isn't in the example!) the latin name of the species
* typeName: If we have a variant of the species (typically for plants), write the name here
* algorithmId: Defining how we calculate the day degrees. 
    1. Basic day degree model with mean temperature and base temperature
	2. Daily heat sum calculated from hourly values based on base temp, max temp, opt temp (TODO: Document the model)
* baseTemp: For the day degree calculation (at least methods 1 and 2)
* upperTemp: For the day degree calculation (at least method 2)
* optTemp: For the day degree calculation (at least method 2)
* startDate: The date (as a Unix timestamp) for when the calculation starts. Also known as biofix
* weatherParams: Array of weather parameters needed for the calculation. TM = Mean temperature
* plantations: Array of development phases of the species, in the correct order
* observations: Array of weather observations

## Output data
Example output data are provided below:

```json
[{
	"validTimeStart": "2020-11-17T00:00:00.000+0000",
	"validTimeEnd": "2020-11-17T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"11.85\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"9.21\",\"WEATHER.TM\":\"24.85\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
}, {
	"validTimeStart": "2020-11-18T00:00:00.000+0000",
	"validTimeEnd": "2020-11-18T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"10.67\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"18.85\",\"WEATHER.TM\":\"23.67\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
}, {
	"validTimeStart": "2020-11-19T00:00:00.000+0000",
	"validTimeEnd": "2020-11-19T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"9.29\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"26.86\",\"WEATHER.TM\":\"22.29\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
}, {
	"validTimeStart": "2020-11-20T00:00:00.000+0000",
	"validTimeEnd": "2020-11-20T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"8.29\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"35.02\",\"WEATHER.TM\":\"21.29\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
}, {
	"validTimeStart": "2020-11-21T00:00:00.000+0000",
	"validTimeEnd": "2020-11-21T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"9.41\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"42.9\",\"WEATHER.TM\":\"22.41\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
}, {
	"validTimeStart": "2020-11-22T00:00:00.000+0000",
	"validTimeEnd": "2020-11-22T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"10.49\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"51.24\",\"WEATHER.TM\":\"23.49\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
}, {
	"validTimeStart": "2020-11-23T00:00:00.000+0000",
	"validTimeEnd": "2020-11-23T00:00:00.000+0000",
	"validGeometry": null,
	"warningStatus": 0,
	"allValues": "{\"GENPHENOLO.HEATREQ\":\"0\",\"THRESHOLD.HATCHED EGG\":\"33.35\",\"THRESHOLD.EGG\":\"0\",\"WEATHER.NON_NEG\":\"10.68\",\"GENPHENOLO.PHASE\":\"Egg\",\"GENPHENOLO.VARIETY\":\"GENERIC\",\"GENPHENOLO.HEATSUM\":\"59.12\",\"WEATHER.TM\":\"23.68\"}",
	"keys": "[\"GENPHENOLO.HEATREQ\",\"THRESHOLD.HATCHED EGG\",\"THRESHOLD.EGG\",\"WEATHER.NON_NEG\",\"GENPHENOLO.PHASE\",\"GENPHENOLO.VARIETY\",\"GENPHENOLO.HEATSUM\",\"WEATHER.TM\"]"
},]
```

The particular point of interest is the GENPHENOLO.PHASE output, which tells the user what the predicted phase of the organism will be at a given time.