/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package no.nibio.vips.model.phenology;

import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.MappingJsonFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import no.nibio.vips.entity.ModelConfiguration;
import no.nibio.vips.entity.Result;
import no.nibio.vips.entity.WeatherObservation;
import no.nibio.vips.model.Model;
import no.nibio.vips.model.ModelId;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.BeforeAll;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.Test;

/**
 *
 * @author wildfly
 */
public class PhenologyModelTest {
    
    private                 String      FILE_WEATHER_DATA_FILE_01       =   "/dataset01/weatherdata_faw_temperature_01_station_id_55.json";
    private                 String      FILE_WEATHER_DATA_FILE_02       =   "/dataset01/weatherdata_faw_temperature_01_station_id_TA00644.json";
    private                 String      FILE_SOW_DATA_FILE_01           =   "/dataset01/sowingData_01.json";
    private                 String      FILE_SOW_DATA_FILE_02           =   "/dataset01/sowingData_02.json";
    private                 String      FILE_SOW_DATA_FILE_04           =   "/dataset01/sowingData_04.json";                    // FAW metadata
    private                 String      FILE_SOW_DATA_FILE_05           =   "/dataset01/sowingData_05.json";                    // FAW metadata
    private                 String      CONST_TIME_ZONE                 =   "Africa/Bamako";
    private                 String      FILE_PLANTATION_01              =   "/dataset01/plantationData_01.json";
    private                 String      FILE_PLANTATION_02              =   "/dataset01/plantationData_02.json";
    private                 String      FILE_PLANTATION_03              =   "/dataset01/plantationData_03.json";
    private                 String      FILE_PLANTATION_04              =   "/dataset01/plantationData_04.json";                // Support of FAW
    
    private final   static  String      NODE_SPECIES                    =   "species";
    private final   static  String      NODE_SPECIES_LATIN_NAME         =   "speciesLatinName";
    private final   static  String      NODE_BASE_TEMP                  =   "baseTemp";
    private final   static  String      NODE_OPT_TEMP                   =   "optTemp";
    private final   static  String      NODE_UPPER_TEMP                 =   "upperTemp";    
    private final   static  String      NODE_LOG_INTERVAL_ID            =   "weatherLogIntervalId";
    private final   static  String      NODE_TYPE_NAMES                 =   "typeNames";
    private final   static  String      NODE_TYPE_NAME                  =   "typeName";
    private final   static  String      NODE_PHASE_INFO                 =   "phaseInfo";
    private final   static  String      NODE_PHASE_NAME                 =   "phaseName";
    private final   static  String      NODE_HEAT_REQ                   =   "heatReq";
    private final   static  String      NODE_WEATHER_PARAMS             =   "weatherParameters";
    private final   static  String      NODE_DD_ALGORITHM_ID            =   "dayDegreeAlgorithmId";
    
    public PhenologyModelTest() {
    }
    
      @BeforeAll
    public static void setUpClass() {
    }
    
    @AfterAll
    public static void tearDownClass() {
    }
    
    @BeforeEach
    public void setUp() {
    }
    
    @AfterEach
    public void tearDown() {
    }

    /**
     * Test of getResult method, of class PhenologyModel.
     */
    @Test
    public void testGetResult() throws Exception {
        System.out.println("getResult");
        ModelConfiguration config   = this.getConfiguration(getWeatherDataFile());
        
        Plant   sowingData  =  getConfigurationSowingData(getSowingDataFile());
        Plant   plantDBData   = getConfigurationPlantation(getPlantationDataFile(), sowingData.getSpeciesLatinName(), sowingData.getTypeName());
        
         if(sowingData.getTypeName().equals(plantDBData.getTypeName()))
        {
            plantDBData.setStartDate(sowingData.getStartDate());
        }
        
        config.setConfigParameter("plantationData", plantDBData);
        
        PhenologyModel instance = new PhenologyModel();
        //System.out.println(config.toJSON());
        instance.setConfiguration(config);
        List<Result> result = instance.getResult();
        
        ObjectMapper objectMapper   = new ObjectMapper();
        objectMapper.writeValue(Paths.get("resultFAW_04.json").toFile(), result);
        //System.out.println(objectMapper.writeValueAsString(result));
        assertNotNull(result);
        
      /* 
        for(Result res:result)
        {
            // LOGGER.log(Level.INFO, res.toString());
            System.out.println(res.toString());
        }
        */
        
    }

    /**
     * Test of getModelId method, of class PhenologyModel.
     */
    @Test
    public void testGetModelId() {
        System.out.println("getModelId");
        PhenologyModel instance = new PhenologyModel();
        ModelId expResult = new ModelId(PhenologyModel.NAME_MODEL_ID);
        ModelId result = instance.getModelId();
        assertEquals(expResult.toString(), result.toString());

    }

    /**
     * Test of getModelName method, of class PhenologyModel.
     */
    @Test
    public void testGetModelName_0args() {
        System.out.println("getModelName");
        PhenologyModel instance = new PhenologyModel();
        String result = instance.getModelName();
        assertNotNull(result);

    }

    /**
     * Test of getModelName method, of class PhenologyModel.
     */
    @Test
    public void testGetModelName_String() {
        System.out.println("getModelName");
        String language = "";
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getModelName(Model.DEFAULT_LANGUAGE);
        assertNotNull(result);
    }

    /**
     * Test of getLicense method, of class PhenologyModel.
     */
    @Test
    public void testGetLicense() {
        System.out.println("getLicense");
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getLicense();
        assertNotNull(result);
    }

    /**
     * Test of getCopyright method, of class PhenologyModel.
     */
    @Test
    public void testGetCopyright() {
        System.out.println("getCopyright");
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getCopyright();
        assertNotNull(result);
    }

    /**
     * Test of getModelDescription method, of class PhenologyModel.
     */
    @Test
    public void testGetModelDescription_0args() {
        System.out.println("getModelDescription");
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getModelDescription();
        assertNotNull(result);
    }

    /**
     * Test of getModelDescription method, of class PhenologyModel.
     */
    @Test
    public void testGetModelDescription_String() {
        System.out.println("getModelDescription");
        String language = "";
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getModelDescription(language);
        assertNotNull(result);
    }

    /**
     * Test of getWarningStatusInterpretation method, of class PhenologyModel.
     */
    @Test
    public void testGetWarningStatusInterpretation_0args() {
        System.out.println("getWarningStatusInterpretation");
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getWarningStatusInterpretation();
        assertNotNull(result);
    }

    /**
     * Test of getWarningStatusInterpretation method, of class PhenologyModel.
     */
    @Test
    public void testGetWarningStatusInterpretation_String() {
        System.out.println("getWarningStatusInterpretation");
        String language = "";
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getWarningStatusInterpretation(language);
        assertNotNull(result);
    }

    /**
     * Test of getModelUsage method, of class PhenologyModel.
     */
    @Test
    public void testGetModelUsage_0args() {
        System.out.println("getModelUsage");
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getModelUsage();
        assertNotNull(result);
    }

    /**
     * Test of getModelUsage method, of class PhenologyModel.
     */
    @Test
    public void testGetModelUsage_String() {
        System.out.println("getModelUsage");
        String language = "";
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getModelUsage(language);
        assertNotNull(result);
    }

    /**
     * Test of getSampleConfig method, of class PhenologyModel.
     */
    @Test
    public void testGetSampleConfig() {
        System.out.println("getSampleConfig");
        PhenologyModel instance = new PhenologyModel();
        String expResult = "";
        String result = instance.getSampleConfig();
        assertNotNull(result);
    }

    /**
     * Test of setConfiguration method, of class PhenologyModel.
     */
    @Test
    public void testSetConfiguration() throws Exception {
        System.out.println("setConfiguration");
        ModelConfiguration config = this.getConfiguration(getWeatherDataFile());
        Plant   sowingData  =  getConfigurationSowingData(getSowingDataFile());
        Plant   plantDBData   = getConfigurationPlantation(getPlantationDataFile(), sowingData.getSpeciesLatinName(), sowingData.getTypeName());
        
        if(sowingData.getTypeName().equals(plantDBData.getTypeName()))
        {
            plantDBData.setStartDate(sowingData.getStartDate());
        }
     
        
        config.setConfigParameter("plantationData", plantDBData);
        
        PhenologyModel instance = new PhenologyModel();
        instance.setConfiguration(config);
        assertNotNull(instance);
    }
  
     private ModelConfiguration getConfiguration(String fileName)
    {
        try {
            
            ModelConfiguration config = new ModelConfiguration();
            config.setModelId(PhenologyModel.MODEL_ID.toString());
            
            
            config.setConfigParameter("timeZone", CONST_TIME_ZONE);
            
            BufferedInputStream inputStream = new BufferedInputStream(this.getClass().getResourceAsStream(fileName));
            JsonFactory f = new MappingJsonFactory();
            JsonParser jp = f.createParser(inputStream);
            JsonNode all = jp.readValueAsTree();
            List<WeatherObservation> observations = new ArrayList<>();
            ObjectMapper mapper = new ObjectMapper();

            Date firstDate = null;
            Date lastDate = null;
            if(all.isArray())
            {
                for(JsonNode node : all){
                    Date timeMeasured = (Date)mapper.convertValue(node.get("timeMeasured").asText(), new TypeReference<Date>(){});
                    if(firstDate == null || firstDate.compareTo(timeMeasured) > 0)
                    {
                        firstDate = timeMeasured;
                    }
                    if(lastDate == null || lastDate.compareTo(timeMeasured) < 0)
                    {
                        lastDate = timeMeasured;
                    }
                    //System.out.println(node.toString());
                    WeatherObservation observation = new WeatherObservation();
                    observation.setTimeMeasured(timeMeasured);
                    observation.setLogIntervalId(node.get("logIntervalId").asInt());
                    observation.setElementMeasurementTypeId(node.get("elementMeasurementTypeId").asText());
                    observation.setValue(node.get("value").asDouble());
                    observations.add(observation);
                }

            }
            else
            {
                fail("Data input from file is not a JSON array");
            }
            
            config.setConfigParameter("observations", observations);
            return config;
        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        } 
    }
    
    private Plant getConfigurationSowingData(String fileName)
    {
        Plant  plant  = new Plant();
        

        try {
            BufferedInputStream inputStream = new BufferedInputStream(this.getClass().getResourceAsStream(fileName));
            JsonFactory f = new MappingJsonFactory();
            JsonParser jp;
            
            jp = f.createParser(inputStream);
            JsonNode all = jp.readValueAsTree();
            
            ObjectMapper mapper = new ObjectMapper(); 
            
            
            plant.setTypeName(all.get("typeName").asText());
            plant.setSpeciesLatinName(all.get("speciesName").asText());
            plant.setStartDate((Date)mapper.convertValue(all.get(DataMatrix.PARAM_BIOFIX).asText(), new TypeReference<Date>(){}));
            
        } catch (IOException ex) {
            Logger.getLogger(PhenologyModelTest.class.getName()).log(Level.SEVERE, null, ex);
        }

        return plant;
    }
 
    private Plant getConfigurationPlantation(String fileName,String strParamSpeciesName, String strParamTypeName)
    {
        Plant   plant   = new Plant();
               
        ObjectMapper        mapper          = new ObjectMapper();
        List<Plantation>    plantations     = new ArrayList<Plantation>();
        try {
            BufferedInputStream inputStream = new BufferedInputStream(this.getClass().getResourceAsStream(fileName));
            JsonFactory f = new MappingJsonFactory();
            JsonParser jp = f.createParser(inputStream);
            
            JsonNode all = jp.readValueAsTree();
            
            JsonNode nodeAllSpecies = all.path(NODE_SPECIES);
            
            if(nodeAllSpecies.isArray())
            {
                for(JsonNode nodeSpecies : nodeAllSpecies)
                {
                    if(nodeSpecies.get(NODE_SPECIES_LATIN_NAME).asText().trim().equalsIgnoreCase(strParamSpeciesName))
                    {
                        plant.setSpeciesLatinName(nodeSpecies.get(NODE_SPECIES_LATIN_NAME).asText());
                        if(null != nodeSpecies.get(NODE_BASE_TEMP))
                        {
                            plant.setBaseTemp(nodeSpecies.get(NODE_BASE_TEMP).asDouble());
                        }
                         if(null != nodeSpecies.get(NODE_OPT_TEMP))
                        {
                            plant.setOptTemp(nodeSpecies.get(NODE_OPT_TEMP).asDouble());
                        }
                        if(null != nodeSpecies.get(NODE_UPPER_TEMP))
                        {
                            plant.setUpperTemp(nodeSpecies.get(NODE_UPPER_TEMP).asDouble());
                        }                          
                        if(null != nodeSpecies.get(NODE_LOG_INTERVAL_ID))
                        {
                            plant.setLogIntervalId(nodeSpecies.get(NODE_LOG_INTERVAL_ID).asInt());
                        }
                        if(null != nodeSpecies.get(NODE_DD_ALGORITHM_ID))
                        {
                            plant.setAlgorithmId(nodeSpecies.get(NODE_DD_ALGORITHM_ID).asInt());
                        }
                       JsonNode nodeSpeciesWeatherParams   = nodeSpecies.path(NODE_WEATHER_PARAMS);
                       if( null != nodeSpeciesWeatherParams && nodeSpeciesWeatherParams.isArray())
                       {
                           String[] strWeatherParams    = new String[nodeSpeciesWeatherParams.size()];
                           int i = 0;
                           for (JsonNode nodeWeather: nodeSpeciesWeatherParams)
                           {
                               strWeatherParams[i++] = mapper.convertValue(nodeWeather,  new TypeReference<String>(){});
                           }
                           plant.setWeatherParams(strWeatherParams);
                       }
                       
                       JsonNode nodeTypeNames       = nodeSpecies.path(NODE_TYPE_NAMES); 
                       
                        if(nodeTypeNames.isArray())
                        {
                             for(JsonNode nodeTypeName : nodeTypeNames)
                             {
                                 if(nodeTypeName.get(NODE_TYPE_NAME).asText().trim().equalsIgnoreCase(strParamTypeName))
                                 {
                                    plant.setTypeName(strParamTypeName);
                                    if(null != nodeTypeName.get(NODE_LOG_INTERVAL_ID))
                                    {
                                        plant.setLogIntervalId(nodeTypeName.get(NODE_LOG_INTERVAL_ID).asInt());
                                    }
                                    if(null != nodeTypeName.get(NODE_DD_ALGORITHM_ID))
                                    {
                                        plant.setAlgorithmId(nodeTypeName.get(NODE_DD_ALGORITHM_ID).asInt());
                                    }                                    
                                    if(null != nodeTypeName.get(NODE_BASE_TEMP))
                                    {
                                        plant.setBaseTemp(nodeTypeName.get(NODE_BASE_TEMP).asDouble());
                                    }
                                    if(null != nodeTypeName.get(NODE_OPT_TEMP)) 
                                    {
                                        plant.setOptTemp(nodeTypeName.get(NODE_OPT_TEMP).asDouble());
                                    }
                                    if(null != nodeTypeName.get(NODE_UPPER_TEMP)) 
                                    {
                                        plant.setUpperTemp(nodeTypeName.get(NODE_UPPER_TEMP).asDouble());
                                    } 
                                    JsonNode nodeTypeNameWeatherParams   = nodeTypeName.path(NODE_WEATHER_PARAMS);
                                    if( null != nodeTypeNameWeatherParams && nodeTypeNameWeatherParams.isArray())
                                        {
                                            String[] strWeatherParams    = new String[nodeTypeNameWeatherParams.size()];
                                            int i = 0;
                                            for (JsonNode nodeWeather: nodeTypeNameWeatherParams)
                                            {
                                                strWeatherParams[i++] = mapper.convertValue(nodeWeather,  new TypeReference<String>(){});
                                            }
                                            plant.setWeatherParams(strWeatherParams);
                                        }
                                    
                                    JsonNode nodeAllPhaseInfo = nodeTypeName.path(NODE_PHASE_INFO);
                                    if(nodeAllPhaseInfo.isArray())
                                    {
                                         for(JsonNode nodePhaseInfo : nodeAllPhaseInfo)
                                         {
                                            Plantation plantation  = new Plantation();
                                            plantation.setTypeName(nodeTypeName.get(NODE_TYPE_NAME).asText());


                                             plantation.setPhaseName(nodePhaseInfo.get(NODE_PHASE_NAME).asText());
                                             plantation.setHeatReq(nodePhaseInfo.get(NODE_HEAT_REQ).asDouble());

                                             plantations.add(plantation);
                                         }

                                    }
                                 } 

                             }

                        }
                       
                       
                    }
                }

            }
     
            plant.setPlantations(plantations);
            
            
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
        }
        
        return plant;
    }
    
    
     private List<Plantation> getConfigurationPlantation02(String fileName)
    {
        List<Plantation> plantations    = new ArrayList<Plantation>();
        
        
        try {
            BufferedInputStream inputStream = new BufferedInputStream(this.getClass().getResourceAsStream(fileName));
            JsonFactory f = new MappingJsonFactory();
            JsonParser jp = f.createParser(inputStream);
            JsonNode all = jp.readValueAsTree();
            JsonNode nodePhaseInfo = all.path("phaseInfo");
            if(nodePhaseInfo.isArray())
            {
                for(JsonNode node : nodePhaseInfo)
                {
                     Plantation   plantation  =   new Plantation
                                                (           all.get("typeName").asText()
                                                        ,   node.get("phaseName").asText()
                                                       ,    node.get("heatReq").asDouble()
                                                );
                     plantations.add(plantation);
                }
            }
            
            
            /*
            if(all.isArray())
            {
               for(JsonNode node : all)
               {
                   Plantation   plantation  =   new Plantation
                                                (
                                                            node.get("maizeVarity").asText()
                                                       ,    node.get("phaseName").asText()
                                                       ,    node.get("baseTemp").asText()
                                                       ,    node.get("heatReq").asDouble()
                                                );
                  plantations.add(plantation);
                  
                } 
            }
            
            */
            
            
        } catch (IOException ex) {
            Logger.getLogger(PhenologyModel.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        return plantations;
    }
     
    
    private String getWeatherDataFile()
    {
        return FILE_WEATHER_DATA_FILE_02;
    }

    private String getSowingDataFile()
    {
        return FILE_SOW_DATA_FILE_05;
    }
    
     private String getPlantationDataFile()
    {
        return FILE_PLANTATION_04;
    }   
    
}
